# State managment seminar

This is a sample todo app.
Demonstrates the [recoil library]().

## Run the mock backend
```sh
cd backend
npm run start
```

## Run the frontend

+ __in separate shell run__
```sh
cd TodoApp
npm i
npm run dev
```

+ do not worry about the errors. You will solve them along the way.

## Complete the todos in the order specified (they are also in code)

To help you navigate the codebase, file needed to be changed are listed.
There are also todos in code to provide you some insights.
Good luck.

### Level 1 thief
1. wrap the component - `main.tsx`
2. define atom for todo list - `State/Atom.ts`
3. initialize value of todo list atom from backend - `App.tsx`
4. add boilerplate JSX to `Components/TodoItem.tsx`
5. Hook up TodoList - `Components/TodoList.tsx`
   1. Make TodoList child of App
   2. retrieve todoList atom value
   3. map todos from Recoil to TodoItem
6. if you have backend running, todos should be now displayed

### Level 50 hitman

1. add filter atom - `State/Atom.ts`
2. add selector utilizing filter and todo atom - `State/Selector.ts`
3. hookup the Filter component - `Filter.tsx` and `App.tsx`
4. change the TodoList atom value retrieval. Now you want to use value from filtered selector and not the raw atom - `Components/TodoList.tsx`
5. you should now be able to filter the todos
6. handle clicking the checkbox - `TodoItem.tsx`
   1. update the state with changed item
   2. /optional/ handle the delete button
7. hookup NewTask - `Components/NewTask.tsx` and `App.tsx`


### Level 100 mafia boss

1. add saving the new task to the db `Components/NewTask.tsx`
   1. handle errors and loading - not in solution but done in previous iteration
2. add stats
   1. create a selector that will count completed, uncompleted and all tasks
   2. create a component that will display the stats
   3. display the stats
      1. think about how nice it is that we can display the stats _anywhere_ since the selector is global
   + no example implementation provided, see lecture slides or [end of this page](https://recoiljs.org/docs/basic-tutorial/selectors)
3. add deleting and saving the progress `Components/TodoItem.tsx`
   + not implemented in solution

### Level 1000 head of five families

1. initializing Atoms from React components is tiresome.jpg
   1. Check out `State/DBEffect.ts`
   2. if you hook it up, and fine tune a bit the component will be initialized from backend automatically
   3. if you add React.Suspence and ErrorBoundary to App.tsx loading and error handling will be handled for you by react (no more state boilerplate yay)
2. saving the tasks on both client and db asynchronously opens pandora box of problems
   1. how would you bring client and db back to sync?
   2. how far do you let user go without connection?
   3. google sheets style - modify client, try to transmit changes to server, if fails try again endlessly (preserve changes on client)
      1. there is an interesting problem with ids. How would you solve it? (uuids)
   4. discord style - internet gone = application gone on first click
3. marking the tasks as done shuffles them. This is terrible UX. Add to the effect above onSet handler and sort the items on every set.
   1. Contemplate why the "sort every time" solution is bad. Come up with better one (sort once, modify smartly)


# Notes for lecturers

1. at the start of the lecture make sure students understand what is state and why is it needed
2. demo is heavily based on the lecture. Make sure students are aware of this and have access to the lecture pdf
3. start easy and complicate the demo slowly. Todos should guide you.

# Sources

+ styles and some logic taken from [here](https://github.com/anilsenay/React-Recoil-Todo-Example)
